<?php


/**
 * Convert to WebP using "cwebp"
 * 
 */


// Do not allow direct access to the file.
if( ! defined( 'ABSPATH' ) ) {
	exit;
}




function stillbe_iqc_extends_conv_cwebp( $original_file, $filename, $size_data = array(), $opt = array() ) {

	// Check Filename
	if( empty( $original_file ) || empty( $filename ) ) {
		return new WP_Error( 'error_webp_filename', __( 'No WebP filename has be passed' ) );
	}

	// Check "-near_lossless" Option
	// Default is FALSE
	$is_near_lossless = false;
	if( apply_filters( 'stillbe_image_quality_control_enable_webp_near_lossless', STILLBE_IQ_ENABLE_WEBP_NEAR_LOSSLESS ) ) {
		$is_near_lossless = stillbe_iqc_extends_chk_near_lossless();
	}

	// Get the Size
	$size = empty( $opt['size'] ) ? null : $opt['size'];
	if( empty( $opt['size'] ) ) {
		return new WP_Error( 'error_getting_size', __( 'No original image size has be passed' ) );
	}

	// Get the Quality
	$webp_quality = empty( $opt['quality'] ) ? 0 : $opt['quality'][0];
	$quality      = empty( $opt['quality'] ) ? 0 : $opt['quality'][1];
	if( empty( $webp_quality * $quality ) ) {
		return new WP_Error( 'error_getting_webp_quality', __( 'No WebP quality has be passed' ) );
	}

	// for Test
	$quality = StillBE_Image_Quality_Ctrl_Setting::chk_num_type( $quality, 'png' );
	if( empty( $quality ) ) {
		$quality = 9;
	}

	// Get the Mime-Type
	$mime_type = empty( $opt['mime'] ) ? 'image/jpeg' : (string) $opt['mime'];
	if( 0 !== strpos( $mime_type, 'image/' ) ) {
		$mime_type = 'image/jpeg';
	}

	// Set Compression Quality Options
	$options = '';
	$_cwebp  = array();
	if( 'image/png' === $mime_type || 'image/gif' === $mime_type ) {
		if( apply_filters( 'stillbe_image_quality_control_enable_webp_lossless_for_png_gif', STILLBE_IQ_ENABLE_WEBP_LOSSLESS ) ) {
			// LossLess or Near-Lossless Compression
			if( $is_near_lossless ) {
				$options .= "-near_lossless {$webp_quality}";
				// Convert Info
				$_cwebp['quality'] = $webp_quality;
				$_cwebp['method']  = 'near-lossless';
			} else {
				if( 'image/gif' === $mime_type ) {
					$quality = 9;
				}
				$_quality  = min( absint( $quality * 10 + 6 ), 100 );   // Quality = 100 in Lossless is Very Time Consuming
				$options .= "-lossless -q {$_quality}";
				// Convert Info
				$_cwebp['q']       = $quality;
				$_cwebp['quality'] = $_quality;
				$_cwebp['method']  = 'lossless';
			}
		} else {
			// Lossy Compression
			$options .= "-q {$webp_quality}";
			// Convert Info
			$_cwebp['quality'] = $webp_quality;
			$_cwebp['method']  = 'lossy';
		}
	} elseif( 'image/jpeg' === $mime_type || 'image/webp' === $mime_type ) {
		// Lossy Compression
		$options .= "-q {$webp_quality}";
		// Convert Info
		$_cwebp['quality'] = $webp_quality;
		$_cwebp['method']  = 'lossy';
	}

	// Additional Options (@since 1.0.0)
	//   -m : Compression method to use (1-6, default = 4)
	//   -mt: Use multi-threading for encoding, if possible.
	$options .= ' -m 6 -mt';

	// Set Crop & Resiez Options
	if( ! isset( $size_data['width'] ) ) {
		$size_data['width'] = $size['width'];
	}
	if ( ! isset( $size_data['height'] ) ) {
		$size_data['height'] = $size['height'];
	}
	if ( ! isset( $size_data['crop'] ) ) {
		$size_data['crop'] = false;
	}

	// Resize
	$_w = 0;
	$_h = 0;
	if( $size['width'] > $size_data['width'] || $size['height'] > $size_data['height'] ) {
		$dims = image_resize_dimensions( $size['width'], $size['height'], $size_data['width'], $size_data['height'], $size_data['crop'] );
		if( ! $dims ) {
			return new WP_Error( 'error_getting_dimensions', __( 'Could not calculate resized image dimensions' ) );
		}
		list( $dst_x, $dst_y, $src_x, $src_y, $dst_w, $dst_h, $src_w, $src_h ) = $dims;
		if( $size_data['crop'] ) {
			$options .= " -crop {$src_x} {$src_y} {$src_w} {$src_h}";
		}
		$options .= " -resize {$dst_w} {$dst_h}";
		$_w = $dst_w;
		$_h = $dst_h;
	}

	// Output Stream
	if( '-' === $filename ) {

		// Convert Image using "cwebp"
		@passthru(
			sprintf(
				'cwebp %s %s -o %s',
				$options,
				$original_file,
				$filename
			),
			$result_code
		);

		// Error Handling
		if( $result_code > 0 ) {
			return new WP_Error( 'error_cwebp_exec', implode( "\n", $output ), $options );
		}

		return $_cwebp;

	}

	// Convert Image using "cwebp"
	@exec(
		sprintf(
			'cwebp %s %s -o %s 2>&1',
			$options,
			$original_file,
			$filename
		),
		$output,
		$result_code
	);

	// Error Handling
	if( $result_code > 0 ) {
		return new WP_Error( 'error_cwebp_exec', implode( "\n", $output ), $options );
	}
	if( ! file_exists( $filename ) ) {
		return new WP_Error( 'error_cwebp_exec', sprintf( __( '%s; File does not exist', 'stillbedomain' ), $filename ) );
	}

	// Return Result
	return array(
		'path'  => $filename,
		'file'  => wp_basename( $filename ),
		'size'  => @filesize( $filename ),
		'q'     => $_cwebp['quality'],
		'cwebp' => $_cwebp,
		'meta'  => array(
			'file'      => wp_basename( apply_filters( 'image_make_intermediate_size', $filename ) ),
			'mime-type' => 'image/webp',
			'width'     => $_w,
			'height'    => $_h,
		),
	);

}






// END of the File



